import { h } from 'vue';

var ElementMixin = {
  inject: ["manager"],
  props: {
    index: {
      type: Number,
      required: true
    },
    disabled: {
      type: Boolean,
      "default": false
    }
  },
  mounted: function mounted() {
    var _this$$props = this.$props,
        disabled = _this$$props.disabled,
        index = _this$$props.index;

    if (!disabled) {
      this.setDraggable(index);
    }
  },
  watch: {
    index: function index(newIndex) {
      if (this.$el && this.$el.sortableInfo) {
        this.$el.sortableInfo.index = newIndex;
      }
    },
    disabled: function disabled(isDisabled) {
      if (isDisabled) {
        this.removeDraggable();
      } else {
        this.setDraggable(this.index);
      }
    }
  },
  beforeUnmount: function beforeUnmount() {
    if (!this.disabled) this.removeDraggable();
  },
  methods: {
    setDraggable: function setDraggable(index) {
      var node = this.$el;
      node.sortableInfo = {
        index: index,
        manager: this.manager
      };
      this.ref = {
        node: node
      };
      this.manager.add(this.ref);
    },
    removeDraggable: function removeDraggable() {
      this.manager.remove(this.ref);
    }
  }
};

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _unsupportedIterableToArray(arr, i) || _nonIterableRest();
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter);
}

function _iterableToArrayLimit(arr, i) {
  if (typeof Symbol === "undefined" || !(Symbol.iterator in Object(arr))) return;
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(o);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

var Manager = /*#__PURE__*/function () {
  function Manager() {
    _classCallCheck(this, Manager);

    this.refs = [];
    this.active = null;
  }

  _createClass(Manager, [{
    key: "add",
    value: function add(ref) {
      if (!this.refs) {
        this.refs = [];
      }

      this.refs.push(ref);
    }
  }, {
    key: "remove",
    value: function remove(ref) {
      var index = this.getIndex(ref);

      if (index !== -1) {
        this.refs.splice(index, 1);
      }
    }
  }, {
    key: "isActive",
    value: function isActive() {
      return !!this.active;
    }
  }, {
    key: "getActive",
    value: function getActive() {
      var _this = this;

      return this.refs.find(function (_ref) {
        var node = _ref.node;
        return node.sortableInfo.index == _this.active.index;
      });
    }
  }, {
    key: "getIndex",
    value: function getIndex(ref) {
      return this.refs.indexOf(ref);
    }
  }, {
    key: "getOrderedRefs",
    value: function getOrderedRefs() {
      return this.refs.sort(function (a, b) {
        return a.node.sortableInfo.index - b.node.sortableInfo.index;
      });
    }
  }]);

  return Manager;
}();

function arrayMove(arr, previousIndex, newIndex) {
  var array = arr.slice(0);

  if (newIndex >= array.length) {
    var k = newIndex - array.length;

    while (k-- + 1) {
      array.push(void 0);
    }
  }

  array.splice(newIndex, 0, array.splice(previousIndex, 1)[0]);
  return array;
}
function arrayRemove(arr, previousIndex) {
  var array = arr.slice(0);
  if (previousIndex >= array.length) return array;
  array.splice(previousIndex, 1);
  return array;
}
function arrayInsert(arr, newIndex, value) {
  var array = arr.slice(0);

  if (newIndex === array.length) {
    array.push(value);
  } else {
    array.splice(newIndex, 0, value);
  }

  return array;
}
var events = {
  start: ["touchstart", "mousedown"],
  move: ["touchmove", "mousemove"],
  end: ["touchend", "touchcancel", "mouseup"]
};
var vendorPrefix = function () {
  if (typeof window === "undefined" || typeof document === "undefined") return "";
  var styles = window.getComputedStyle(document.documentElement, "") || ["-moz-hidden-iframe"];
  var pre = (Array.prototype.slice.call(styles).join("").match(/-(moz|webkit|ms)-/) || styles.OLink === "" && ["", "o"])[1];

  switch (pre) {
    case "ms":
      return "ms";

    default:
      return pre && pre.length ? pre[0].toUpperCase() + pre.substr(1) : "";
  }
}();
function closest(el, fn) {
  while (el) {
    if (fn(el)) return el;
    el = el.parentNode;
  }
}
function limit(min, max, value) {
  if (value < min) {
    return min;
  }

  if (value > max) {
    return max;
  }

  return value;
}

function getCSSPixelValue(stringValue) {
  if (stringValue.substr(-2) === "px") {
    return parseFloat(stringValue);
  }

  return 0;
}

function getElementMargin(element) {
  var style = window.getComputedStyle(element);
  return {
    top: getCSSPixelValue(style.marginTop),
    right: getCSSPixelValue(style.marginRight),
    bottom: getCSSPixelValue(style.marginBottom),
    left: getCSSPixelValue(style.marginLeft)
  };
}
function getPointerOffset(e) {
  var reference = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : "page";
  var x = "".concat(reference, "X");
  var y = "".concat(reference, "Y");
  return {
    x: e.touches ? e.touches[0][x] : e[x],
    y: e.touches ? e.touches[0][y] : e[y]
  };
}

function offsetParents(node) {
  var nodes = [node];

  for (; node; node = node.offsetParent) {
    nodes.unshift(node);
  }

  return nodes;
}

function commonOffsetParent(node1, node2) {
  var parents1 = offsetParents(node1);
  var parents2 = offsetParents(node2);
  if (parents1[0] != parents2[0]) throw "No common ancestor!";

  for (var i = 0; i < parents1.length; i++) {
    if (parents1[i] != parents2[i]) return parents1[i - 1];
  }
}
function getEdgeOffset(node, container) {
  var offset = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {
    top: 0,
    left: 0
  };

  if (node) {
    var nodeOffset = {
      top: offset.top + node.offsetTop,
      left: offset.left + node.offsetLeft
    };

    if (node.offsetParent !== container.offsetParent) {
      return getEdgeOffset(node.offsetParent, container, nodeOffset);
    } else {
      return nodeOffset;
    }
  }
}
function cloneNode(node) {
  var fields = node.querySelectorAll("input, textarea, select");
  var clonedNode = node.cloneNode(true);

  var clonedFields = _toConsumableArray(clonedNode.querySelectorAll("input, textarea, select"));

  clonedFields.forEach(function (field, index) {
    if (field.type !== "file" && fields[index]) {
      field.value = fields[index].value;
    }
  });
  return clonedNode;
}
function getLockPixelOffsets(lockOffset, width, height) {
  if (!Array.isArray(lockOffset)) {
    lockOffset = [lockOffset, lockOffset];
  }

  if (lockOffset.length !== 2) {
    throw new Error("lockOffset prop of SortableContainer should be a single value or an array of exactly two values. Given ".concat(lockOffset));
  }

  var _lockOffset = lockOffset,
      _lockOffset2 = _slicedToArray(_lockOffset, 2),
      minLockOffset = _lockOffset2[0],
      maxLockOffset = _lockOffset2[1];

  return [getLockPixelOffset(minLockOffset, width, height), getLockPixelOffset(maxLockOffset, width, height)];
}
function getLockPixelOffset(lockOffset, width, height) {
  var offsetX = lockOffset;
  var offsetY = lockOffset;
  var unit = "px";

  if (typeof lockOffset === "string") {
    var match = /^[+-]?\d*(?:\.\d*)?(px|%)$/.exec(lockOffset);

    if (match === null) {
      throw new Error("lockOffset value should be a number or a string of a number followed by \"px\" or \"%\". Given ".concat(lockOffset));
    }

    offsetX = offsetY = parseFloat(lockOffset);
    unit = match[1];
  }

  if (!isFinite(offsetX) || !isFinite(offsetY)) {
    throw new Error("lockOffset value should be a finite. Given ".concat(lockOffset));
  }

  if (unit === "%") {
    offsetX = offsetX * width / 100;
    offsetY = offsetY * height / 100;
  }

  return {
    x: offsetX,
    y: offsetY
  };
}
function getDistance(x1, y1, x2, y2) {
  var x = x1 - x2;
  var y = y1 - y2;
  return Math.sqrt(x * x + y * y);
}
function getRectCenter(clientRect) {
  return {
    x: clientRect.left + clientRect.width / 2,
    y: clientRect.top + clientRect.height / 2
  };
}
function resetTransform() {
  var nodes = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

  for (var i = 0, len = nodes.length; i < len; i++) {
    var node = nodes[i];
    var el = node.node;
    if (!el) return;
    node.edgeOffset = null;
    el.style["".concat(vendorPrefix, "Transform")] = "";
    el.style["".concat(vendorPrefix, "TransitionDuration")] = "";
  }
}

var ContainerMixin = {
  inject: ["SlicksortHub"],
  emits: ["sort-start", "sort-move", "sort-end", "sort-insert", "sort-remove", "update:list"],
  data: function data() {
    var useHub = false;
    var containerId = 1;

    if (this.group) {
      if (this.SlicksortHub) {
        useHub = true;
        containerId = this.SlicksortHub.getId();
      } else if (process.env.NODE_ENV !== "production") {
        throw new Error('Slicksort plugin required to use "group" prop');
      }
    }

    return {
      sorting: false,
      id: containerId,
      hub: useHub ? this.SlicksortHub : null,
      manager: new Manager(),
      events: {
        start: this.handleStart,
        move: this.handleMove,
        end: this.handleEnd
      }
    };
  },
  props: {
    list: {
      type: Array,
      required: true
    },
    axis: {
      type: String,
      "default": "y"
    },
    distance: {
      type: Number,
      "default": 0
    },
    pressDelay: {
      type: Number,
      "default": 0
    },
    pressThreshold: {
      type: Number,
      "default": 5
    },
    useDragHandle: {
      type: Boolean,
      "default": false
    },
    useWindowAsScrollContainer: {
      type: Boolean,
      "default": false
    },
    hideSortableGhost: {
      type: Boolean,
      "default": true
    },
    lockToContainerEdges: {
      type: Boolean,
      "default": false
    },
    lockOffset: {
      type: [String, Number, Array],
      "default": "50%"
    },
    transitionDuration: {
      type: Number,
      "default": 300
    },
    appendTo: {
      type: String,
      "default": "body"
    },
    draggedSettlingDuration: {
      type: Number,
      "default": null
    },
    group: {
      type: String,
      "default": ""
    },
    accept: {
      type: [Boolean, Array, Function],
      "default": null
    },
    block: {
      type: Array,
      "default": function _default() {
        return [];
      }
    },
    lockAxis: String,
    helperClass: String,
    contentWindow: Object,
    shouldCancelStart: {
      type: Function,
      "default": function _default(e) {
        var disabledElements = ["input", "textarea", "select", "option", "button"];
        return disabledElements.indexOf(e.target.tagName.toLowerCase()) !== -1;
      }
    },
    getHelperDimensions: {
      type: Function,
      "default": function _default(_ref) {
        var node = _ref.node;
        return {
          width: node.offsetWidth,
          height: node.offsetHeight
        };
      }
    }
  },
  provide: function provide() {
    return {
      manager: this.manager
    };
  },
  mounted: function mounted() {
    var _this = this;

    this.container = this.$el;
    this.document = this.container.ownerDocument || document;
    this._window = this.contentWindow || window;
    this.scrollContainer = this.useWindowAsScrollContainer ? this.document.body : this.container;

    var _loop = function _loop(key) {
      if (_this.events.hasOwnProperty(key)) {
        events[key].forEach(function (eventName) {
          return _this.container.addEventListener(eventName, _this.events[key]);
        });
      }
    };

    for (var key in this.events) {
      _loop(key);
    }

    if (this.hub) {
      this.hub.addContainer(this);
    }
  },
  beforeUnmount: function beforeUnmount() {
    var _this2 = this;

    var _loop2 = function _loop2(key) {
      if (_this2.events.hasOwnProperty(key)) {
        events[key].forEach(function (eventName) {
          return _this2.container.removeEventListener(eventName, _this2.events[key]);
        });
      }
    };

    for (var key in this.events) {
      _loop2(key);
    }

    if (this.hub) {
      this.hub.removeContainer(this);
    }
  },
  methods: {
    handleStart: function handleStart(e) {
      var _this3 = this;

      var _this$$props = this.$props,
          distance = _this$$props.distance,
          shouldCancelStart = _this$$props.shouldCancelStart;

      if (e.button === 2 || shouldCancelStart(e)) {
        return false;
      }

      this._touched = true;
      this._pos = getPointerOffset(e);
      var node = closest(e.target, function (el) {
        return el.sortableInfo != null;
      });

      if (node && node.sortableInfo && this.nodeIsChild(node) && !this.sorting) {
        var useDragHandle = this.$props.useDragHandle;
        var index = node.sortableInfo.index;
        if (useDragHandle && !closest(e.target, function (el) {
          return el.sortableHandle != null;
        })) return;
        this.manager.active = {
          index: index
        };

        if (e.target.tagName.toLowerCase() === "a") {
          e.preventDefault();
        }

        if (!distance) {
          if (this.pressDelay === 0) {
            this.handlePress(e);
          } else {
            this.pressTimer = setTimeout(function () {
              return _this3.handlePress(e);
            }, this.pressDelay);
          }
        }
      }
    },
    nodeIsChild: function nodeIsChild(node) {
      return node.sortableInfo.manager === this.manager;
    },
    handleMove: function handleMove(e) {
      var _this$$props2 = this.$props,
          distance = _this$$props2.distance,
          pressThreshold = _this$$props2.pressThreshold;

      if (!this.sorting && this._touched) {
        var offset = getPointerOffset(e);
        this._delta = {
          x: this._pos.x - offset.x,
          y: this._pos.y - offset.y
        };
        var delta = Math.abs(this._delta.x) + Math.abs(this._delta.y);

        if (!distance && (!pressThreshold || pressThreshold && delta >= pressThreshold)) {
          clearTimeout(this.cancelTimer);
          this.cancelTimer = setTimeout(this.cancel, 0);
        } else if (distance && delta >= distance && this.manager.isActive()) {
          this.handlePress(e);
        }
      }
    },
    handleEnd: function handleEnd() {
      if (!this._touched) return;
      var distance = this.$props.distance;
      this._touched = false;

      if (!distance) {
        this.cancel();
      }
    },
    cancel: function cancel() {
      if (!this.sorting) {
        clearTimeout(this.pressTimer);
        this.manager.active = null;

        if (this.hub) {
          this.hub.cancel();
        }
      }
    },
    handlePress: function handlePress(e) {
      var _this4 = this;

      e.stopPropagation();
      var active = this.manager.getActive();

      if (active) {
        var _this$$props3 = this.$props,
            getHelperDimensions = _this$$props3.getHelperDimensions,
            helperClass = _this$$props3.helperClass,
            hideSortableGhost = _this$$props3.hideSortableGhost,
            appendTo = _this$$props3.appendTo;
        var node = active.node;
        var index = node.sortableInfo.index;
        var margin = getElementMargin(node);
        var containerBoundingRect = this.container.getBoundingClientRect();
        var dimensions = getHelperDimensions({
          index: index,
          node: node
        });
        this.node = node;
        this.margin = margin;
        this.width = dimensions.width;
        this.height = dimensions.height;
        this.marginOffset = {
          x: this.margin.left + this.margin.right,
          y: Math.max(this.margin.top, this.margin.bottom)
        };
        this.boundingClientRect = node.getBoundingClientRect();
        this.containerBoundingRect = containerBoundingRect;
        this.index = index;
        this.newIndex = index;
        var clonedNode = cloneNode(node);
        this.helper = this.document.querySelector(appendTo).appendChild(clonedNode);
        this.helper.style.position = "fixed";
        this.helper.style.top = "".concat(this.boundingClientRect.top - margin.top, "px");
        this.helper.style.left = "".concat(this.boundingClientRect.left - margin.left, "px");
        this.helper.style.width = "".concat(this.width, "px");
        this.helper.style.height = "".concat(this.height, "px");
        this.helper.style.boxSizing = "border-box";
        this.helper.style.pointerEvents = "none";

        if (hideSortableGhost) {
          this.sortableGhost = node;
        }

        if (this.hub) {
          this.hub.sortStart({
            ref: this
          });
          this.hub.helper = this.helper;
          this.hub.ghost = this.sortableGhost;
        }

        this.intializeOffsets(e, this.boundingClientRect);
        this.offsetEdge = getEdgeOffset(node, this.container);

        if (helperClass) {
          var _this$helper$classLis;

          (_this$helper$classLis = this.helper.classList).add.apply(_this$helper$classLis, _toConsumableArray(helperClass.split(" ")));
        }

        this.listenerNode = e.touches ? node : this._window;
        events.move.forEach(function (eventName) {
          return _this4.listenerNode.addEventListener(eventName, _this4.handleSortMove);
        });
        events.end.forEach(function (eventName) {
          return _this4.listenerNode.addEventListener(eventName, _this4.handleSortEnd);
        });
        this.sorting = true;
        this.$emit("sort-start", {
          event: e,
          node: node,
          index: index
        });
      }
    },
    handleSortMove: function handleSortMove(e) {
      e.preventDefault();
      this.updatePosition(e);

      if (this.hub) {
        var payload = this.list[this.index];
        this.hub.handleSortMove(e, payload);
      }

      if (!this.hub || this.hub.isDest(this)) {
        this.animateNodes();
        this.autoscroll();
      }

      this.$emit("sort-move", {
        event: e
      });
    },
    handleDropOut: function handleDropOut() {
      var removed = this.list[this.index];
      var newValue = arrayRemove(this.list, this.index);
      this.$emit("sort-remove", {
        oldIndex: this.index
      });
      this.$emit("update:list", newValue);
      return removed;
    },
    handleDropIn: function handleDropIn(item) {
      var newValue = arrayInsert(this.list, this.newIndex, item);
      this.$emit("sort-insert", {
        newIndex: this.newIndex,
        value: item
      });
      this.$emit("update:list", newValue);
      this.handleDragOut();
    },
    handleDragOut: function handleDragOut() {
      if (this.autoscrollInterval) {
        clearInterval(this.autoscrollInterval);
        this.autoscrollInterval = null;
      }

      if (this.hub.isSource(this)) {
        this.animateNodes();
      } else {
        resetTransform(this.manager.refs);
        this.sortableGhost.remove();
        this.manager.active = null;
        this._touched = false;
        this.sorting = false;
      }
    },
    intializeOffsets: function intializeOffsets(e, clientRect) {
      var useWindowAsScrollContainer = this.useWindowAsScrollContainer,
          containerBoundingRect = this.containerBoundingRect,
          _window = this._window;
      this.marginOffset = {
        x: this.margin.left + this.margin.right,
        y: Math.max(this.margin.top, this.margin.bottom)
      };
      this._axis = {
        x: this.axis.indexOf("x") >= 0,
        y: this.axis.indexOf("y") >= 0
      };
      this.initialOffset = getPointerOffset(e);
      this.initialScroll = {
        top: this.scrollContainer.scrollTop,
        left: this.scrollContainer.scrollLeft
      };
      this.initialWindowScroll = {
        top: window.pageYOffset,
        left: window.pageXOffset
      };
      this.translate = {};
      this.minTranslate = {};
      this.maxTranslate = {};

      if (this._axis.x) {
        this.minTranslate.x = (useWindowAsScrollContainer ? 0 : containerBoundingRect.left) - clientRect.left - this.width / 2;
        this.maxTranslate.x = (useWindowAsScrollContainer ? _window.innerWidth : containerBoundingRect.left + containerBoundingRect.width) - clientRect.left - this.width / 2;
      }

      if (this._axis.y) {
        this.minTranslate.y = (useWindowAsScrollContainer ? 0 : containerBoundingRect.top) - clientRect.top - this.height / 2;
        this.maxTranslate.y = (useWindowAsScrollContainer ? _window.innerHeight : containerBoundingRect.top + containerBoundingRect.height) - clientRect.top - this.height / 2;
      }
    },
    handleDragIn: function handleDragIn(e, sortableGhost, helper) {
      if (this.hub.isSource(this)) {
        return;
      }

      var nodes = this.manager.refs;
      this.index = nodes.length;
      this.manager.active = {
        index: this.index
      };
      var containerBoundingRect = this.container.getBoundingClientRect();
      var helperBoundingRect = helper.getBoundingClientRect();
      this.containerBoundingRect = containerBoundingRect;
      this.sortableGhost = cloneNode(sortableGhost);
      this.container.appendChild(this.sortableGhost);
      var ghostRect = this.sortableGhost.getBoundingClientRect();
      this.boundingClientRect = ghostRect;
      this.margin = getElementMargin(this.sortableGhost);
      this.width = ghostRect.width;
      this.height = ghostRect.height;
      this.offsetEdge = getEdgeOffset(this.sortableGhost, this.container);
      this.intializeOffsets(e, ghostRect);
      this.initialOffset.x += ghostRect.x - helperBoundingRect.x;
      this.initialOffset.y += ghostRect.y - helperBoundingRect.y;
      this.sorting = true;
    },
    handleSortEnd: function handleSortEnd(e) {
      var _this5 = this;

      if (this.listenerNode) {
        events.move.forEach(function (eventName) {
          return _this5.listenerNode.removeEventListener(eventName, _this5.handleSortMove);
        });
        events.end.forEach(function (eventName) {
          return _this5.listenerNode.removeEventListener(eventName, _this5.handleSortEnd);
        });
      }

      var nodes = this.manager.refs;

      var onEnd = function onEnd() {
        _this5.helper.parentNode.removeChild(_this5.helper);

        if (_this5.hideSortableGhost && _this5.sortableGhost) {
          _this5.sortableGhost.style.visibility = "";
          _this5.sortableGhost.style.opacity = "";
        }

        resetTransform(nodes);
        clearInterval(_this5.autoscrollInterval);
        _this5.autoscrollInterval = null;

        if (_this5.hub && !_this5.hub.isDest(_this5)) {
          _this5.hub.handleSortEnd(_this5.index);
        } else {
          _this5.$emit("sort-end", {
            event: e,
            oldIndex: _this5.index,
            newIndex: _this5.newIndex
          });

          _this5.$emit("update:list", arrayMove(_this5.list, _this5.index, _this5.newIndex));
        }

        _this5.manager.active = null;
        _this5._touched = false;
        _this5.sorting = false;
      };

      if (this.transitionDuration || this.draggedSettlingDuration) {
        this.transitionHelperIntoPlace(nodes, onEnd);
      } else {
        onEnd();
      }
    },
    transitionHelperIntoPlace: function transitionHelperIntoPlace(nodes, cb) {
      var _this6 = this;

      if (this.draggedSettlingDuration === 0 || nodes.length === 0) {
        return Promise.resolve();
      }

      var indexNode = nodes[this.index].node;
      var targetX = 0;
      var targetY = 0;
      var scrollDifference = {
        top: window.pageYOffset - this.initialWindowScroll.top,
        left: window.pageXOffset - this.initialWindowScroll.left
      };

      if (this.hub && !this.hub.isDest(this)) {
        var dest = this.hub.getDest();
        var destIndex = dest.newIndex;
        var destRefs = dest.manager.getOrderedRefs();
        var destNode = destIndex < destRefs.length ? destRefs[destIndex].node : dest.sortableGhost;
        var ancestor = commonOffsetParent(indexNode, destNode);
        var sourceOffset = getEdgeOffset(indexNode, ancestor);
        var targetOffset = getEdgeOffset(destNode, ancestor);
        targetX = targetOffset.left - sourceOffset.left - scrollDifference.left;
        targetY = targetOffset.top - sourceOffset.top - scrollDifference.top;
      } else {
        var newIndexNode = nodes[this.newIndex].node;
        var deltaScroll = {
          left: this.scrollContainer.scrollLeft - this.initialScroll.left + scrollDifference.left,
          top: this.scrollContainer.scrollTop - this.initialScroll.top + scrollDifference.top
        };
        targetX = -deltaScroll.left;

        if (this.translate && this.translate.x > 0) {
          targetX += newIndexNode.offsetLeft + newIndexNode.offsetWidth - (indexNode.offsetLeft + indexNode.offsetWidth);
        } else {
          targetX += newIndexNode.offsetLeft - indexNode.offsetLeft;
        }

        targetY = -deltaScroll.top;

        if (this.translate && this.translate.y > 0) {
          targetY += newIndexNode.offsetTop + newIndexNode.offsetHeight - (indexNode.offsetTop + indexNode.offsetHeight);
        } else {
          targetY += newIndexNode.offsetTop - indexNode.offsetTop;
        }
      }

      var duration = this.draggedSettlingDuration !== null ? this.draggedSettlingDuration : this.transitionDuration;
      this.helper.style["".concat(vendorPrefix, "Transform")] = "translate3d(".concat(targetX, "px,").concat(targetY, "px, 0)");
      this.helper.style["".concat(vendorPrefix, "TransitionDuration")] = "".concat(duration, "ms");

      var cleanup = function cleanup(event) {
        if (!event || event.propertyName === "transform") {
          clearTimeout(cleanupTimer);
          resetTransform([_this6.helper]);
          cb();
        }
      };

      var cleanupTimer = setTimeout(cleanup, duration + 10);
      this.helper.addEventListener("transitionend", cleanup);
    },
    updatePosition: function updatePosition(e) {
      var _this$$props4 = this.$props,
          lockAxis = _this$$props4.lockAxis,
          lockToContainerEdges = _this$$props4.lockToContainerEdges;
      var offset = getPointerOffset(e);
      var translate = {
        x: offset.x - this.initialOffset.x,
        y: offset.y - this.initialOffset.y
      };
      translate.y -= window.pageYOffset - this.initialWindowScroll.top;
      translate.x -= window.pageXOffset - this.initialWindowScroll.left;
      this.translate = translate;

      if (lockToContainerEdges) {
        var _getLockPixelOffsets = getLockPixelOffsets(this.lockOffset, this.height, this.width),
            _getLockPixelOffsets2 = _slicedToArray(_getLockPixelOffsets, 2),
            minLockOffset = _getLockPixelOffsets2[0],
            maxLockOffset = _getLockPixelOffsets2[1];

        var minOffset = {
          x: this.width / 2 - minLockOffset.x,
          y: this.height / 2 - minLockOffset.y
        };
        var maxOffset = {
          x: this.width / 2 - maxLockOffset.x,
          y: this.height / 2 - maxLockOffset.y
        };
        translate.x = limit(this.minTranslate.x + minOffset.x, this.maxTranslate.x - maxOffset.x, translate.x);
        translate.y = limit(this.minTranslate.y + minOffset.y, this.maxTranslate.y - maxOffset.y, translate.y);
      }

      if (lockAxis === "x") {
        translate.y = 0;
      } else if (lockAxis === "y") {
        translate.x = 0;
      }

      if (this.helper) {
        this.helper.style["".concat(vendorPrefix, "Transform")] = "translate3d(".concat(translate.x, "px,").concat(translate.y, "px, 0)");
      }
    },
    animateNodes: function animateNodes() {
      var _this$$props5 = this.$props,
          transitionDuration = _this$$props5.transitionDuration,
          hideSortableGhost = _this$$props5.hideSortableGhost;
      var nodes = this.manager.getOrderedRefs();
      var deltaScroll = {
        left: this.scrollContainer.scrollLeft - this.initialScroll.left,
        top: this.scrollContainer.scrollTop - this.initialScroll.top
      };
      var sortingOffset = {
        left: this.offsetEdge.left + this.translate.x + deltaScroll.left,
        top: this.offsetEdge.top + this.translate.y + deltaScroll.top
      };
      var scrollDifference = {
        top: window.pageYOffset - this.initialWindowScroll.top,
        left: window.pageXOffset - this.initialWindowScroll.left
      };
      this.newIndex = null;

      for (var i = 0, len = nodes.length; i < len; i++) {
        var node = nodes[i].node;
        var index = node.sortableInfo.index;
        var width = node.offsetWidth;
        var height = node.offsetHeight;
        var offset = {
          width: this.width > width ? width / 2 : this.width / 2,
          height: this.height > height ? height / 2 : this.height / 2
        };
        var translate = {
          x: 0,
          y: 0
        };
        var edgeOffset = nodes[i].edgeOffset;

        if (!edgeOffset) {
          nodes[i].edgeOffset = edgeOffset = getEdgeOffset(node, this.container);
        }

        var nextNode = i < nodes.length - 1 && nodes[i + 1];
        var prevNode = i > 0 && nodes[i - 1];

        if (nextNode && !nextNode.edgeOffset) {
          nextNode.edgeOffset = getEdgeOffset(nextNode.node, this.container);
        }

        if (index === this.index) {
          if (hideSortableGhost) {
            this.sortableGhost = node;
            node.style.visibility = "hidden";
            node.style.opacity = 0;
          }

          continue;
        }

        if (transitionDuration) {
          node.style["".concat(vendorPrefix, "TransitionDuration")] = "".concat(transitionDuration, "ms");
        }

        if (this._axis.x) {
          if (this._axis.y) {
            if (index < this.index && (sortingOffset.left + scrollDifference.left - offset.width <= edgeOffset.left && sortingOffset.top + scrollDifference.top <= edgeOffset.top + offset.height || sortingOffset.top + scrollDifference.top + offset.height <= edgeOffset.top)) {
              translate.x = this.width + this.marginOffset.x;

              if (edgeOffset.left + translate.x > this.containerBoundingRect.width - offset.width) {
                translate.x = nextNode.edgeOffset.left - edgeOffset.left;
                translate.y = nextNode.edgeOffset.top - edgeOffset.top;
              }

              if (this.newIndex === null) {
                this.newIndex = index;
              }
            } else if (index > this.index && (sortingOffset.left + scrollDifference.left + offset.width >= edgeOffset.left && sortingOffset.top + scrollDifference.top + offset.height >= edgeOffset.top || sortingOffset.top + scrollDifference.top + offset.height >= edgeOffset.top + height)) {
              translate.x = -(this.width + this.marginOffset.x);

              if (edgeOffset.left + translate.x < this.containerBoundingRect.left + offset.width) {
                translate.x = prevNode.edgeOffset.left - edgeOffset.left;
                translate.y = prevNode.edgeOffset.top - edgeOffset.top;
              }

              this.newIndex = index;
            }
          } else {
            if (index > this.index && sortingOffset.left + scrollDifference.left + offset.width >= edgeOffset.left) {
              translate.x = -(this.width + this.marginOffset.x);
              this.newIndex = index;
            } else if (index < this.index && sortingOffset.left + scrollDifference.left <= edgeOffset.left + offset.width) {
              translate.x = this.width + this.marginOffset.x;

              if (this.newIndex == null) {
                this.newIndex = index;
              }
            }
          }
        } else if (this._axis.y) {
          if (index > this.index && sortingOffset.top + scrollDifference.top + offset.height >= edgeOffset.top) {
            translate.y = -(this.height + this.marginOffset.y);
            this.newIndex = index;
          } else if (index < this.index && sortingOffset.top + scrollDifference.top <= edgeOffset.top + offset.height) {
            translate.y = this.height + this.marginOffset.y;

            if (this.newIndex == null) {
              this.newIndex = index;
            }
          }
        }

        node.style["".concat(vendorPrefix, "Transform")] = "translate3d(".concat(translate.x, "px,").concat(translate.y, "px,0)");
      }

      if (this.newIndex == null) {
        this.newIndex = this.index;
      }
    },
    autoscroll: function autoscroll() {
      var _this7 = this;

      var translate = this.translate;
      var direction = {
        x: 0,
        y: 0
      };
      var speed = {
        x: 1,
        y: 1
      };
      var acceleration = {
        x: 10,
        y: 10
      };

      if (translate.y >= this.maxTranslate.y - this.height / 2) {
        direction.y = 1;
        speed.y = acceleration.y * Math.abs((this.maxTranslate.y - this.height / 2 - translate.y) / this.height);
      } else if (translate.x >= this.maxTranslate.x - this.width / 2) {
        direction.x = 1;
        speed.x = acceleration.x * Math.abs((this.maxTranslate.x - this.width / 2 - translate.x) / this.width);
      } else if (translate.y <= this.minTranslate.y + this.height / 2) {
        direction.y = -1;
        speed.y = acceleration.y * Math.abs((translate.y - this.height / 2 - this.minTranslate.y) / this.height);
      } else if (translate.x <= this.minTranslate.x + this.width / 2) {
        direction.x = -1;
        speed.x = acceleration.x * Math.abs((translate.x - this.width / 2 - this.minTranslate.x) / this.width);
      }

      if (this.autoscrollInterval) {
        clearInterval(this.autoscrollInterval);
        this.autoscrollInterval = null;
      }

      if (direction.x !== 0 || direction.y !== 0) {
        this.autoscrollInterval = setInterval(function () {
          var offset = {
            left: 1 * speed.x * direction.x,
            top: 1 * speed.y * direction.y
          };
          _this7.scrollContainer.scrollTop += offset.top;
          _this7.scrollContainer.scrollLeft += offset.left;
          _this7.translate.x += offset.left;
          _this7.translate.y += offset.top;

          _this7.animateNodes();
        }, 5);
      }
    }
  }
};

var HandleDirective = {
  beforeMount: function beforeMount(el) {
    el.sortableHandle = true;
  }
};

function create(name, defaultTag, mixin) {
  return {
    name: name,
    mixins: [mixin],
    props: {
      tag: {
        type: String,
        "default": defaultTag
      }
    },
    render: function render() {
      return h(this.tag, this.$slots["default"]());
    }
  };
}

var SlickList = create("slick-list", "div", ContainerMixin);
var SlickItem = create("slick-item", "div", ElementMixin);

var containerIDCounter = 1;

function canAcceptElement(dest, source, payload) {
  if (source.id === dest.id) return true;
  if (dest.block && dest.block.includes(source.group)) return false;

  if (typeof dest.accept === "function") {
    return dest.accept({
      dest: dest,
      source: source,
      payload: payload
    });
  }

  if (typeof dest.accept === "boolean") {
    return dest.accept;
  }

  if (dest.accept && dest.accept.includes(source.group)) return true;
  if (dest.group === source.group) return true;
  return false;
}

function findClosestDest(_ref, refs) {
  var x = _ref.x,
      y = _ref.y;
  var closest = null;
  var minDistance = Infinity;

  for (var i = 0; i < refs.length; i++) {
    var ref = refs[i];
    var center = getRectCenter(ref.container.getBoundingClientRect());
    var distance = getDistance(x, y, center.x, center.y);

    if (distance < minDistance) {
      closest = ref;
      minDistance = distance;
    }
  }

  return closest;
}

var SlicksortHub = /*#__PURE__*/function () {
  function SlicksortHub() {
    _classCallCheck(this, SlicksortHub);

    this.refs = [];
    this.sorting = false;
    this.helper = null;
    this.ghost = null;
  }

  _createClass(SlicksortHub, [{
    key: "getId",
    value: function getId() {
      return "" + containerIDCounter++;
    }
  }, {
    key: "isSource",
    value: function isSource(_ref2) {
      var id = _ref2.id;
      return this.source.id === id;
    }
  }, {
    key: "getSource",
    value: function getSource() {
      return this.source;
    }
  }, {
    key: "isDest",
    value: function isDest(_ref3) {
      var id = _ref3.id;
      return this.dest.id === id;
    }
  }, {
    key: "getDest",
    value: function getDest() {
      return this.dest;
    }
  }, {
    key: "addContainer",
    value: function addContainer(ref) {
      this.refs.push(ref);
    }
  }, {
    key: "removeContainer",
    value: function removeContainer(ref) {
      this.refs = this.refs.filter(function (c) {
        return c.id !== ref.id;
      });
    }
  }, {
    key: "sortStart",
    value: function sortStart(_ref4) {
      var ref = _ref4.ref;
      this.sorting = true;
      this.source = ref;
      this.dest = ref;
    }
  }, {
    key: "handleSortMove",
    value: function handleSortMove(e, payload) {
      var dest = this.dest;
      var source = this.source;
      var refs = this.refs;
      var pointer = getPointerOffset(e, "client");
      var newDest = findClosestDest(pointer, refs);

      if (dest.id !== newDest.id && canAcceptElement(newDest, source, payload)) {
        this.dest = newDest;
        dest.handleDragOut(e);
        newDest.handleDragIn(e, this.ghost, this.helper);
      }

      if (dest.id !== this.source.id) {
        this.dest.updatePosition(e);
        this.dest.animateNodes();
        this.dest.autoscroll();
      }
    }
  }, {
    key: "handleSortEnd",
    value: function handleSortEnd() {
      if (this.source.id === this.dest.id) return;
      var payload = this.source.handleDropOut();
      this.dest.handleDropIn(payload);
      this.reset();
    }
  }, {
    key: "reset",
    value: function reset() {
      this.source = null;
      this.dest = null;
      this.helper = null;
      this.ghost = null;
      this.sorting = false;
    }
  }, {
    key: "cancel",
    value: function cancel(e) {
      this.dest.handleDragOut(e);
      this.source.handleSortEnd(e);
      this.reset();
    }
  }]);

  return SlicksortHub;
}();

var plugin = {
  install: function install(app, options) {
    app.directive("drag-handle", HandleDirective);
    app.provide("SlicksortHub", new SlicksortHub(options));
  }
};

export { ContainerMixin, ElementMixin, HandleDirective, SlickItem, SlickList, arrayMove, plugin };
